@@ -1,36 +1,24 @@
-# The following comment should be removed at some point in the future.
-# mypy: strict-optional=False
-
-from __future__ import absolute_import
-
 import contextlib
-import errno
 import hashlib
 import logging
 import os
+from types import TracebackType
+from typing import Dict, Iterator, Optional, Set, Type, Union
 
-from pipenv.patched.notpip._vendor import contextlib2
-
+from pipenv.patched.notpip._internal.models.link import Link
+from pipenv.patched.notpip._internal.req.req_install import InstallRequirement
 from pipenv.patched.notpip._internal.utils.temp_dir import TempDirectory
-from pipenv.patched.notpip._internal.utils.typing import MYPY_CHECK_RUNNING
-
-if MYPY_CHECK_RUNNING:
-    from types import TracebackType
-    from typing import Dict, Iterator, Optional, Set, Type, Union
-    from pipenv.patched.notpip._internal.req.req_install import InstallRequirement
-    from pipenv.patched.notpip._internal.models.link import Link
 
 logger = logging.getLogger(__name__)
 
 
 @contextlib.contextmanager
-def update_env_context_manager(**changes):
-    # type: (str) -> Iterator[None]
+def update_env_context_manager(**changes: str) -> Iterator[None]:
     target = os.environ
 
     # Save values from the target and change them.
     non_existent_marker = object()
-    saved_values = {}  # type: Dict[str, Union[object, str]]
+    saved_values: Dict[str, Union[object, str]] = {}
     for name, new_value in changes.items():
         try:
             saved_values[name] = target[name]
@@ -51,10 +39,9 @@ def update_env_context_manager(**changes):
 
 
 @contextlib.contextmanager
-def get_requirement_tracker():
-    # type: () -> Iterator[RequirementTracker]
+def get_requirement_tracker() -> Iterator["RequirementTracker"]:
     root = os.environ.get('PIP_REQ_TRACKER')
-    with contextlib2.ExitStack() as ctx:
+    with contextlib.ExitStack() as ctx:
         if root is None:
             root = ctx.enter_context(
                 TempDirectory(kind='req-tracker')
@@ -66,38 +53,34 @@ def get_requirement_tracker():
             yield tracker
 
 
-class RequirementTracker(object):
+class RequirementTracker:
 
-    def __init__(self, root):
-        # type: (str) -> None
+    def __init__(self, root: str) -> None:
         self._root = root
-        self._entries = set()  # type: Set[InstallRequirement]
+        self._entries: Set[InstallRequirement] = set()
         logger.debug("Created build tracker: %s", self._root)
 
-    def __enter__(self):
-        # type: () -> RequirementTracker
+    def __enter__(self) -> "RequirementTracker":
         logger.debug("Entered build tracker: %s", self._root)
         return self
 
     def __exit__(
         self,
-        exc_type,  # type: Optional[Type[BaseException]]
-        exc_val,  # type: Optional[BaseException]
-        exc_tb  # type: Optional[TracebackType]
-    ):
-        # type: (...) -> None
+        exc_type: Optional[Type[BaseException]],
+        exc_val: Optional[BaseException],
+        exc_tb: Optional[TracebackType]
+    ) -> None:
         self.cleanup()
 
-    def _entry_path(self, link):
-        # type: (Link) -> str
+    def _entry_path(self, link: Link) -> str:
         hashed = hashlib.sha224(link.url_without_fragment.encode()).hexdigest()
         return os.path.join(self._root, hashed)
 
-    def add(self, req):
-        # type: (InstallRequirement) -> None
+    def add(self, req: InstallRequirement) -> None:
         """Add an InstallRequirement to build tracking.
         """
 
+        assert req.link
         # Get the file to write information about this requirement.
         entry_path = self._entry_path(req.link)
 
@@ -106,45 +89,42 @@ class RequirementTracker(object):
         try:
             with open(entry_path) as fp:
                 contents = fp.read()
-        except IOError as e:
-            # if the error is anything other than "file does not exist", raise.
-            if e.errno != errno.ENOENT:
-                raise
+        except FileNotFoundError:
+            pass
         else:
-            message = '%s is already being built: %s' % (req.link, contents)
+            message = '{} is already being built: {}'.format(
+                req.link, contents)
             raise LookupError(message)
 
         # If we're here, req should really not be building already.
         assert req not in self._entries
 
         # Start tracking this requirement.
-        with open(entry_path, 'w') as fp:
+        with open(entry_path, 'w', encoding="utf-8") as fp:
             fp.write(str(req))
         self._entries.add(req)
 
         logger.debug('Added %s to build tracker %r', req, self._root)
 
-    def remove(self, req):
-        # type: (InstallRequirement) -> None
+    def remove(self, req: InstallRequirement) -> None:
         """Remove an InstallRequirement from build tracking.
         """
 
+        assert req.link
         # Delete the created file and the corresponding entries.
         os.unlink(self._entry_path(req.link))
         self._entries.remove(req)
 
         logger.debug('Removed %s from build tracker %r', req, self._root)
 
-    def cleanup(self):
-        # type: () -> None
+    def cleanup(self) -> None:
         for req in set(self._entries):
             self.remove(req)
 
         logger.debug("Removed build tracker: %r", self._root)
 
     @contextlib.contextmanager
-    def track(self, req):
-        # type: (InstallRequirement) -> Iterator[None]
+    def track(self, req: InstallRequirement) -> Iterator[None]:
         self.add(req)
         yield
         self.remove(req)
